#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
cf-configuration-exporter
"""

__program__ = "cf-configuration-exporter"
__version__ = "0.0.9"
__author__ = "Claudio Benfatto"
__year__ = "2017"
__email__ = "<claudio.benfatto@springer.com>"
__license__ = "MIT"
